package co.yixiang.yshop.module.order.handle;

import cn.hutool.core.thread.ExecutorBuilder;
import cn.hutool.core.thread.ThreadFactoryBuilder;
import cn.hutool.core.util.StrUtil;
import co.yixiang.yshop.framework.common.constant.ShopConstants;
import co.yixiang.yshop.module.order.service.storeaftersales.StoreAfterSalesService;
import co.yixiang.yshop.module.order.service.storeorder.AppStoreOrderService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RBlockingDeque;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadFactory;

/**
 * 延时队列消费
 *
 * @author hupeng
 * @date 2023.7.27
 */
@Component
@Slf4j
public class RedisDelayHandle {
    @Resource
    private RedissonClient redissonClient;
    @Resource
    private AppStoreOrderService appStoreOrderService;

    @Resource
    private StoreAfterSalesService storeAfterSalesService;


    @PostConstruct
    public void startJobTimer() {
        ThreadFactory threadFactory = new ThreadFactoryBuilder().setNamePrefix("delay-job-service").build();
        log.info("========延时队列开始=========");
        ExecutorService executorService = ExecutorBuilder.create()
                .setCorePoolSize(4)
                .setMaxPoolSize(10)
                .setKeepAliveTime(0)
                .setThreadFactory(threadFactory)
                .build();

        executorService.execute(new ExecutorTaskUnPay());
        executorService.execute(new ExecutorTaskUnConfirm());
        executorService.execute(new ExecutorTaskAfterSalesConfirm());
    }

    class ExecutorTaskUnPay implements Runnable {
        @SneakyThrows
        @Override
        public void run() {
            RBlockingDeque<String> blockingUnPayDeque = redissonClient
                    .getBlockingDeque(ShopConstants.REDIS_ORDER_OUTTIME_UNPAY_QUEUE);
            while (true) {
                try {
                    log.info("======延时取消订单循环任务=====");
                    String unPayOrderId  = blockingUnPayDeque.take();
                    log.info("获取到未付款订单id：{}", unPayOrderId);
                    if (StrUtil.isNotEmpty(unPayOrderId)) {
                        appStoreOrderService.cancelOrder(unPayOrderId, null);
                    }
                } catch (Exception e) {
                    log.error("延时取消订单消费失败，"+e.getMessage());
                }
            }
        }
    }

    class ExecutorTaskUnConfirm implements Runnable {
        @SneakyThrows
        @Override
        public void run() {
            RBlockingDeque<String> blockingUnConfirmDeque = redissonClient
                    .getBlockingDeque(ShopConstants.REDIS_ORDER_OUTTIME_UNCONFIRM);
            while (true) {
                log.info("======延时确认收货循环任务=====");
                String unConfirmOrderId = "";
                try {
                    unConfirmOrderId = blockingUnConfirmDeque.take();
                    log.info("获取到待确认收货订单id：{}", unConfirmOrderId);
                    if (StrUtil.isNotEmpty(unConfirmOrderId)) {
                        appStoreOrderService.takeOrder(unConfirmOrderId, null);
                    }
                } catch (Exception e) {
                    log.error("延时确认收货失败，"+e.getMessage());
                }
            }
        }
    }

    class ExecutorTaskAfterSalesConfirm implements Runnable {
        @SneakyThrows
        @Override
        public void run() {
            RBlockingDeque<Long> blockingUnConfirmDeque = redissonClient
                    .getBlockingDeque(ShopConstants.REDIS_AFTERSALESORDER_UNCONFIRM);
            while (true) {
                log.info("======延时售后用户确认收货循环任务=====");
                Long unConfirmOrderId;
                try {
                    unConfirmOrderId = blockingUnConfirmDeque.take();
                    log.info("获取到待确认售后用户收货订单id：{}", unConfirmOrderId);
                    storeAfterSalesService.taskAfterSalesConfirm(unConfirmOrderId);
                } catch (Exception e) {
                    log.error("延时确认售后用户收货失败，"+e.getMessage());
                }
            }
        }
    }

}
